/*
  BV523 TFT Colour display interface
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  The source code is not available at this time, just the compiled
  libraray (TFT320.o) and this head file. For documentation see:
  www.doc.byvac.com

*/ 
#ifndef _H_lcdTFT320_h
#define _H_lcdTFT320_h

#include "types.h"     

// Constants for this display
#define     MAXx    240
#define     MAXy    320

// Command constants - not including set up
#define COL_ADDRESS_START2 0x02
#define COL_ADDRESS_START1 0x03
#define COL_ADDRESS_END2   0x04
#define COL_ADDRESS_END1   0x05
#define ROW_ADDRESS_START2 0x06
#define ROW_ADDRESS_START1 0x07
#define ROW_ADDRESS_END2   0x08
#define ROW_ADDRESS_END1   0x09
#define VERT_SCROLL_START2 0x42 //0x14
#define VERT_SCROLL_START1 0x41 //0x15
#define GATE_SCAN_CONTROL  0x18  // turns scrolling on/off
#define MEM_ACCESS 0x11 //0x16
#define SRAM_WRITE 0x22

// *****************************************************************************   
// Hardware interface, see also hardware initialisation  
// *****************************************************************************   
#define b(X) (1<<X)   
#define LCD_RESET_1()     LATESET=b(1)   
#define LCD_RESET_0()     LATECLR=b(1)   
#define LCD_CS_1()        LATCSET=b(15)   
#define LCD_CS_0()        LATCCLR=b(15)   
#define LCD_RS_1()        LATCSET=b(12)   
#define LCD_RS_0()        LATCCLR=b(12)   
#define LCD_WR_1()        LATCSET=b(13)   
#define LCD_WR_0()        LATCCLR=b(13)   
#define LCD_RD_1()        LATCSET=b(14)   
#define LCD_RD_0()        LATCCLR=b(14) 
// Touch ports
#define D_CLK_1()         LATDSET=b(8)
#define D_CLK_0()         LATDCLR=b(8)
#define D_CS_1()          LATDSET=b(9)
#define D_CS_0()          LATDCLR=b(9)
#define D_IN_1()          LATDSET=b(10) // input to controller
#define D_IN_0()          LATDCLR=b(10)
//#define D_BUSY()          PORTD&0x100
#define D_OUT()           PORTD&0x800 // bit 11
//#define D_PT()            PORTD&0x400 
// back  light
#define LCD_BL_OFF()      LATESET=b(2)   
#define LCD_BL_ON()       LATECLR=b(2)   
 

     

//Format Conversion Logic   
#define RGB565(RED,GREEN,BLUE)  ((((RED&0x03F)>>1)<<11)|((GREEN&0x03F)<<5)|((BLUE&0x03F)>>1)) 
#define SWAP(a, b)  { a ^= b; b ^= a; a ^= b; } // use with care  
// Significantly reduces time to refresh the whole screen from 54ms to 40ms
#define HX_DATA(x) { LCD_RS_1(); LCD_CS_0();  \
                    LATDCLR=0xff; LATDSET=x&0xff; \
                    LATBCLR=0xff00; LATBSET=x&0xff00; \
                    LCD_WR_0(); LCD_WR_1(); LCD_CS_1(); } 
//

// specific types
#define COLOUR WORD     // colours specified as 16 bits
#define BLACK RGB565(0,0,0)
#define WHITE RGB565(63,63,63)
//#define PRESSURE_TOL 10 // key touch pressure tolerence
// ============ Touch screen calibration ==============
typedef struct {
	WORD x;
	WORD y;
} POINT;

typedef struct {
	LONG An;
	LONG Bn;
	LONG Cn;
	LONG Dn;
	LONG En;
	LONG Fn;
	LONG Divider;
} MATRIX;

// ============== Display structures =============================
// set up used for switching between portrate and landsacep,
// these variables are filled at initialisation
typedef struct {
    // general defaults
    COLOUR    foreground;     // foreground colour
    COLOUR    background;     // background colour
    // orientation parameters
    BYTE    landscape;  // keeps track of which vars stored
    WORD    size_x;
    WORD    size_y;
    WORD    cpos_x;     // keeps current pixel position
    WORD    cpos_y;
    // Font parametrs
	BYTE    *font;		// points to font table
	BYTE    *(*getCharData)(BYTE); // gets char data from font
	BYTE    width;		// font width -- Average width
	BYTE    aWidth;    // acual width
	BYTE    height;	// font height
	WORD    nBytes; // number of bytes in a char, can exceed 256  
    // Cursor
    BYTE    crON; 
    // Touch
    BYTE    readings; // number of readings to take for each touch
    MATRIX  calibration;
} HXLCD;


// *****************************************************************************
// variables used throughout   
// *****************************************************************************
// ================ Variables =======================================
//static DWORD crTick;       // cursor tick counter
static BYTE  scrollingF;   // system flag used in scrolling
static SBYTE  crSusF=0;       // suspends cursor when drawing
static HXLCD hx;           // Display parameters


// Helper words
HXLCD *hx_screen();

void hx_initialise();
void hx_set_defaultFont();
void hx_cursor(BYTE on);
void hx_setForeground(COLOUR cl);
void hx_setBackground(COLOUR cl);
void hx_setLandscape(BYTE yes);
void hx_delay(DWORD t);
void hx_cmd(WORD cmd);
void hx_data_old(WORD data);	
WORD hx_read();
void hx_write_reg(WORD addr,WORD val);   
WORD hx_read_reg(WORD addr);   
void hx_allScreen();
void hx_hw_init();   
void hx_reset();
void hx_cls(WORD d1); 
void hx_move(WORD x, WORD y);
void hx_getCoords(WORD *x, WORD *y);
void hx_setAddress(WORD x, WORD y);
void hx_setRect(WORD x0, WORD y0, WORD x1, WORD y1);
void hx_putPixel(WORD x,WORD y,WORD color);
WORD hx_getPixel(WORD x,WORD y);
void hx_bl(BYTE oon); 

// ========= Graphics ====================
void hx_line(WORD x0, WORD y0, WORD x1, WORD y1, WORD colour);
void hx_box(WORD x0, WORD y0, WORD x1, WORD y1, WORD colour);
void hx_rectangle(WORD x0, WORD y0, WORD x1, WORD y1, BYTE fill, WORD colour);
void hx_circle(WORD x0, WORD y0, WORD radius, WORD colour);

// ========= Images ====================
void hx_bitmap(BYTE (*getchp)(), WORD x, WORD y);

// ========= Fonts & Text ================
BYTE *get_char_data(BYTE c);
void hx_setFont(BYTE *font, BYTE*(*get_char_data_function)(BYTE));
void hx_setUserFont(BYTE *fontData, BYTE*(*get_char_data_function)(BYTE));
BYTE hx_render(WORD x, WORD y, BYTE *cp, WORD fore, WORD back);
void hx_out(BYTE c);
void hx_putc(BYTE c);
WORD hx_puts(char *s);

// =========== Scrolling & Cursor ========
void hx_crVis(unsigned char on);
void hx_crSus(BYTE off);
void hx_crd(SHORT lines);
void hx_crr(BYTE places);
void hx_crl(BYTE places);
void hx_cru(BYTE places);
void hx_crBegin();
void hx_crHome();
void hx_crlf(); 

// ========= Touch ========================
WORD hx_t_adc(BYTE cmd);
void hx_t_rawxy(POINT *tc);
void hx_t_scans(BYTE value);
BYTE hx_t_xy(WORD *x, WORD *y);
void hx_t_calibrate();
void hx_t_getCal(MATRIX *cal);
void hx_t_setCal(MATRIX cal);
BYTE hx_t_hit(WORD xt, WORD yt, WORD x, WORD y, WORD x1, WORD y1);


#endif

