
#include <plib.h>
#include "proj2.h"
#include "TFT320.h"
#include "fonts\Arial_Unicode_MS.h"

// ******************************************************************
// Processor initialisation 
// ******************************************************************
void proc_init(void)
{
int Fpb;		// clock reteuned value from start up

	// Configure the device for maximum performance. 
	// This macro sets flash wait states, PBCLK divider and DRM wait states based on the specified
	// clock frequency. It also turns on the cache mode if avaialble.
	// We're using the PB clock frequency that was set automatically by this call.
	// Another option is to set the PB divider explicitely and then use a #define for the pbClk value.
	// **** perif clock is set to SYSCLK (72 or 80Mhz) / 2) = 36MHz
	Fpb=SYSTEMConfigPerformance(CCLK);
	DDPCONbits.JTAGEN=0; // disable jtag

}


// ******************************************************************
// blocking delay near enough 1ms
// ******************************************************************
void delayMS(int delay)
{
int j;

    while(delay--)
        for(j=0;j<7692;j++);
}

// ******************************************************************
// example 1, clearing screen
// ******************************************************************
void ex1_clear()
{
unsigned char r=0,g=0,b=0;
	for(r=0;r<63;r++)
		hx_cls(RGB565(r,g,b));
	r=0;
	for(g=0;g<63;g++)
		hx_cls(RGB565(r,g,b));
	g=0;
	r=0;
	for(b=0;b<63;b++)
		hx_cls(RGB565(r,g,b));
}	

// ******************************************************************
// example of using different fonts
// ******************************************************************
void ex2_fonts()
{
	// installs the font into structure
	hx_setFont(Arial_Unicode_MS,get_char_data);
	hx_cls(RGB565(63,63,63));
	hx_setBackground(RGB565(63,63,63)); // same as cls
	hx_setForeground(RGB565(0,0,0)); // black
	hx_puts("\nHello");
}	

// ******************************************************************
// touch screen example
// ******************************************************************
void ex3_touch()
{
WORD x,y;
	hx_cls(0xffff); // white
    hx_crSus(1); // turn cursor off
    hx_puts("touch screen pen\n");
    while(1) {
        if(hx_t_press()) {
            hx_t_xy(&x,&y);
            hx_putPixel(x,y,RGB565(0,0,0));
    	}
	}
}	

// ******************************************************************
// main, this is where all C programs start
// ******************************************************************
int main()
{
	    hx_initialise(); // initialise display
	    hx_puts("\nScreen initialised..\r"); // on display
	    // examples
	    //ex1_clear();
	    //ex2_fonts();
	    //ex3_touch();	
}	
